from Products import ZCatalog
from Products.ZDBase import ZDiscussions

from Globals import HTMLFile, MessageDialog
from DateTime import DateTime
from OFS.Folder import Folder
from ImageFile import ImageFile
from FileObject import FileObject

import regsub, ts_regex
from string import strip, split, rfind, lower, join, atoi

#crlf=ts_regex.compile('\r\n\|\n\r')
#dpat=ts_regex.compile('[\\/]')
crlf='\r\n\|\n\r'
dpat='[\\/]'

class ZDConferaItem(ZDiscussions.ZDItem):
    """This class provides some compatability functions to help
    make it looks like a Confera Message/Reply"""

    # Allow (reluctantly) access to unprotected attributes
    __allow_access_to_unprotected_subobjects__=1

    meta_type = 'Message'
    icon = 'misc_/ZDConfera/message.gif'
    root = 0

    manage_editForm = HTMLFile('editMessageForm', globals())
    manage = manage_main = manage_editForm

    def __init__(self, id, thread, title, author, body, email, notify,
		 reviewed, file):
	if type(id) != type(''):
	    id = `id`
	self.id = id
	if len(thread):
	    self.in_reply_to = thread[-1].id
	else:
	    self.in_reply_to = ''
	self.title = title

	self.author = author
	self.email = email

	self.body = body
	self.notify = notify
	self.reviewed = reviewed
	self.created = self.modified = DateTime()
	if file:
	    setattr(self, file._name, file)
	    self.file = file
	else:
	    self.file = ''
	self.revsub = 0
    
    def __len__(self): return 1

    # __getitem__ is free, I think.  However, Confera takes care to re-wrap
    # objects in it's aquisition context.  Is this significant?  I think it's
    # done for free.

    # Confera defines a setItem which removes the aquisition wrapper.
    # Is this significant?

    # delItem is a bit complicated because it has to notify everything
    # with a reference to it that it is leaving.  This isn't necisary for ZD.
    # It also monkeys with revsub.  I haven't determined if I need to use
    # revsub at all, so I'm going to wait in this method.

    messageValues = ZDiscussions.ZDItem.children  # Is order significant?

    # A lot of the next batch of methods are lifted directly out of Confera
    # Some may already be provided by inherited methods, but oh well.

    def tpId(self):
	return self.id

    def tpURL(self):
	return self.id

    def this(self):
	return (self,)

    def has_items(self):
	return len(self.children())

    def my_topic(self):
	while self.aq_parent.meta_type != 'Confera Topic':
	    self = self.aq_parent
	return self.aq_parent

    # sub_ids is an internal function, I think.

    def date_created(self):
	parts = self.created.parts()
	return '%d/%d/%d' % (parts[1], parts[2],  parts[0])
    
    def time_created(self):
	parts = self.created.parts()
	return '%02d:%02d' % (parts[3], parts[4])

    def attachment(self):
	file = self.file
	return file and (file,) or None

    def suggest_title(self):
	t = self.title
	return (lower(t[:3])=='re:') and t or 'Re: %s' % t

    def topic_url(self):
	obj = self
	while not obj.root:
	    obj = obj.aq_parent
	return obj.absolute_url()

    def thread_path(self):
	return '/' + join(map(lambda x: x.id, self.ancestors()), '/')

    def index_html(self, REQUEST):
	""" """
	# Why was it done this way?
	return self.message_html(self, REQUEST)

    def doNotify(self, msg, REQUEST):
	if self.notify and self.email and self.mail_host:
	    mail =self.mail_html(self, REQUEST, newItem=(msg,))
	    mhost=getattr(self,self.mail_host)
	    mhost.send(mail)

    def cancelNotify(self, REQUEST):
	""" """
	self.notify=''
	return MessageDialog(title='Cancelled Notification',
			     message='You will no longer be notified of ' \
			     'replies to this message',
			     action=self.topic_url())

    def addMessage(self, title, author, body, email='', notify='', file='',
		   submit='Cancel', REQUEST=None, RESPONSE=None):
	""" """
	#if submit=='Cancel':
	#    RESPONSE.redirect('%s/index_html' % self.topic_url())
	title   =strip(title)
	author  =strip(author)
	email   =strip(email)
	body    =split(regsub.gsub(crlf,'\n',body),'\n')
	notify  =notify
	reviewed=(not self.moderated) and 1 or 0

	try:    file=FileObject(file)
	except: file=''

	if notify and not email:
	    return MessageDialog(
		title='Data Missing',
		message='You must enter an email address for notification!',
		action=REQUEST['URL1']+'/addMessageForm')
	if not title or not author or not body:
	    return MessageDialog(
		title='Data Missing',
		message='You must enter a title, author and body!',
		action=REQUEST['URL1']+'/addMessageForm')

	id=self.createId()
	now = DateTime()
	for parent in self.ancestors() + [self]:
	    parent.modified=now
	    if not reviewed:
		parent.revsub = parent.revsub + 1
		    
	msg = ZDConferaItem(id, self.ancestors() + [self], title, author,
       			 body, email, notify, reviewed, file)
	msg.meta_type = 'Reply'
	msg.icon = 'misc_/ZDConfera/reply.gif'


	self.my_topic()._setObject(id, msg)

	if REQUEST:
	    try:    self.doNotify(msg,REQUEST)
	    except: pass

	    p='%s%s' % (REQUEST['SCRIPT_NAME'],REQUEST['PATH_INFO'])
	    t=self.topic()[0].id
	    p=p[:(rfind(p,t)+len(t))]
	    e=(DateTime('GMT') + 365).rfc822()
	    resp=REQUEST['RESPONSE']
	    resp.setCookie('suggest_author',author,path=p,expires=e)
	    resp.setCookie('suggest_email',email,path=p,expires=e)
	    resp.setCookie('suggest_notify',notify,path=p,expires=e)
	    return MessageDialog(title='Message Posted',
				 message='Your message has been posted',
				 action=p)

    def manage_edit(self,title,author,body,email='',notify='',reviewed='',
		    REQUEST=None,RESPONSE=None):
	""" """
	if not title or not author or not body:
	    return MessageDialog(
		title='Data Missing',
		message='You must enter a title, author and body!',
		action=REQUEST['URL1']+'/manage_editForm')
	if notify and not email:
	    return MessageDialog(
		title='Data Missing',
		message='You must enter an email address for notification!',
		action=REQUEST['URL1']+'/manage_editForm')
	self.title =strip(title)
	self.author=strip(author)
	self.email =strip(email)
	self.body  =split(regsub.gsub(crlf,'\n',body),'\n')
	self.notify=notify

	if self.moderated and (not self.reviewed) and reviewed:
	    self.reviewed=1
	    for obj in self.ancestors():
		if obj.revsub > 0: obj.revsub=obj.revsub-1
	if RESPONSE:
	    RESPONSE.redirect('%s/manage_messages' % self.topic_url())

class ZDConferaReply(ZDConferaItem):
    """Compatibility class"""

class ZDConferaTopic(ZDiscussions.ZDTopic):
    """Compatibility class"""

    meta_type = 'Confera Topic'
    description = 'Confera Topic'
    icon = 'misc_/ZDConfera/topic.gif'
    root = 1

    manage_messages = HTMLFile('Topic_manage_messages', globals())
    manage_editForm = HTMLFile('Topic_editForm', globals())

    manage_options = (
	{ 'label':'Contents', 'icon':icon, 'action':'manage_main', 'target':'manage_main' },
	{ 'label':'Messages', 'icon':'', 'action':'manage_messages', 'target':'manage_main' },
	{ 'label':'Options', 'icon':'', 'action':'manage_editForm', 'target':'manage_main' },
	{ 'label':'Properties', 'icon':'', 'action':'manage_propertiesForm', 'target':'manage_main' },
	{ 'label':'Security', 'icon':'', 'action':'manage_access', 'target':'manage_main' },
	{ 'label':'Undo', 'icon':'', 'action':'manage_UndoForm','target':'manage_main' },
	)

    __ac_permissions__=Folder.__ac_permissions__+(
	('Manage messages', ['manage_edit', 'manage_delete']),
	('Add messages', ['addMessage'], ('Anonymous', 'Manager')),
	)

    def __init__(self, id, title, mhost, exp, mod, default):
	ZDiscussions.ZDTopic.__init__(self, id, title)

	now = DateTime()
	self.created = now
	self.modified = now
	self.mail_host = mhost
	self.expire = exp
	self.moderated = mod

	if default:
	    self.defaultDocFile('index_html','Topic Homepage',
				'ZDConfera/Topic_index_html')
	    self.defaultDocFile('message_html','Message',
				'ZDConfera/Topic_message_html')
	    self.defaultDocFile('searchForm','Search Form',
				'ZDConfera/Topic_searchForm')
	    self.defaultDocFile('confera_searchResults','Search Results',
				'ZDConfera/Topic_searchResults')
	    self.defaultDocFile('mail_html','Notification Email',
				'ZDConfera/Topic_mail_html')
	    self.defaultDocFile('addMessageForm','Add Message Form',
				'ZDConfera/addMessageForm')

    def __len__(self): return 1

    def createId(self):
	id = int(DateTime().timeTime())
	while hasattr(self, str(id)):
	    id=id+1
	return str(id)

    def defaultDocFile(self, id, title, file):
	f=open('%s/Products/%s.dtml' % (SOFTWARE_HOME, file))
	file = f.read()
	f.close()
	self.manage_addDTMLMethod(id, title, file)

    messageValues = ZDiscussions.ZDTopic.posts

    def objectItems(self):
	# Ugly hack to prevent messages from being in contents window
	oI = ZDiscussions.ZDTopic.objectItems(self)
	oI = filter(lambda x: not x[1].__class__ == ZDConferaItem, oI)
	return oI

    def tpId(self):
	return self.id

    def tpURL(self):
	return self.id

    def this(self):
	return self

    def topic(self):
	return (self,)

    def topic_url(self):
        return self.REQUEST['URL1']

    def has_items(self):
	return len(self.posts())

    # Does has_items ever return a value different than item_count's?

    item_count = has_items

    def mailhost_list(self):
	try:
	    return self.superValues(('Mail Host',))
	except:
	    return []

    def expire_items(self):
	if self.expire:
	    t = DateTime() - self.expire
	    for item in filter(lambda x, t=t: x.modified < t, self.posts()):
		try:
		    self.delItem(item.id)
		except:
		    pass
	return ''

    def addMessage(self, title, author, body, email='', notify='', file='',
		   submit='Cancel', REQUEST=None, RESPONSE=None):
	""" """
	#if submit=='Cancel':
	#    RESPONSE.redirect('%s/index_html' % self.topic_url())
	title =strip(title)
	author=strip(author)
	email =strip(email)
	body  =split(regsub.gsub(crlf,'\n',body),'\n')
	notify=notify
	reviewed=(not self.moderated) and 1 or 0

	try:    file=FileObject(file)
	except: file=''

	if notify and not email:
	    return MessageDialog(
		title='Data Missing',
		message='You must enter an email address for notification!',
		action=REQUEST['URL1']+'/addMessageForm')
	
	if not title or not author or not body:
	    return MessageDialog(
		title='Data Missing',
		message='You must enter a title, author and body!',
		action=REQUEST['URL1']+'/addMessageForm')

	id=self.createId()
	msg=ZDConferaItem(id,[],title,author,body,email,notify,reviewed,file)
	self._setObject(id, msg)

	self.expire_items()

	if REQUEST:
	    p='%s%s' % (REQUEST['SCRIPT_NAME'],REQUEST['PATH_INFO'])
	    t=self.id
	    p=p[:(rfind(p,t)+len(t))]
	    e=(DateTime('GMT') + 365).rfc822()
	    resp=REQUEST['RESPONSE']
	    resp.setCookie('suggest_author',author,path=p,expires=e)
	    resp.setCookie('suggest_email',email,path=p,expires=e)
	    resp.setCookie('suggest_notify',notify,path=p,expires=e)
	    return MessageDialog(title='Message Posted',
				 message='Your message has been posted',
				 action=REQUEST['URL1'])
    def search(self, REQUEST):
	""" """
	sr=self.__call__(REQUEST,1)
	rc=len(sr)
	return self.confera_searchResults(self,REQUEST,search_results=sr,
					  result_count=rc)

    def manage_edit(self, mailhost='', exp=0, expire='0', moderated=0,
		    REQUEST=None):
	""" """
	if exp:
	    expire=atoi(expire)
	else:
	    expire = 0
	if mailhost:
	    try:
		v=getattr(self, mailhost)
	    except:
		return MessageDialog(title='Invalid Mail Host',
				     message='Cannot find the named mail host!',
				     action=REQUEST['URL']+'/manage_main')
	self.mail_host = mailhost
	self.expire = expire
	self.moderated = moderated
	return self.manage_main(self, REQUEST)

    def manage_delete(self, ids=[], REQUEST=None):
	"""Resistance is futile!!"""
	for id in ids:
	    try:
		msg = getattr(self, id)
		children = msg.children()
		if children:
		    self.manage_delete(map(lambda x: x.id, children))
		if self.moderated and not msg.reviewed:
		    for obj in msg.ancestors():
			if obj.revsub > 0: obj.revsub = obj.revsub - 1
	    except: pass
	    self._delObject(id)

	if REQUEST:
	    REQUEST.RESPONSE.redirect(REQUEST['URL1'] + '/manage_messages')

    # The components of the searchable interface I think I need to provide.
    # Would it be better just to leave ZCatalog's methods exposed?

    def __call__(self, REQUEST=None, internal=0):
	results = self.searchResults(REQUEST)
	results = map(lambda x,self=self,r=REQUEST:
		       self.getobject(x.data_record_id_, r),
		      results)
	if internal:
	    return results
	return ResultSet(r)

    def _searchable_result_columns(self):
	return ({'name':'title', 'type':'s'},
		{'name':'author','type':'s'},
		{'name':'body',  'type':'s'},
		{'name':'email', 'type':'s'},
		)    

    def _searchable_arguments(self):
	return {'author': {'type':'string', 'optional':1, 'indexed':1},
		'body':   {'type':'string', 'optional':0, 'indexed':1},
		}

    
class ResultItem:
    def __init__(self,data):
        self._data=data

    _schema=['title','author','body','email']

    def __getattr__(self,key):
        if key=='body':
            return join(self._data.body,'<BR>')
        return getattr(self._data, key)

    def __getitem__(self,key,inttype=type(0)):
        try:
            if type(key) is inttype:
                return getattr(self._data,self._schema[key])
            return getattr(self._data, key)
        except (IndexError, KeyError):
            raise AttributeError, key

    def __len__(self):
        return len(self._schema)

class ResultSet:
    def __init__(self,data):  
        self._data=map(ResultItem, data)

    def __getitem__(self,index):
        return self._data[index]
    
    def __len__(self):
        return len(self._data)

    def __getslice__(self,i1,i2):
        return self._data[i1:i2]

    slice=__getslice__
    _searchable_result_columns=ZDConferaTopic._searchable_result_columns
